use std::io::{BufRead, BufReader, Write};
use std::net::{TcpListener, TcpStream};
use std::thread;

pub fn start_tcp_command_server() {
    let listener = TcpListener::bind("127.0.0.1:9999").unwrap();
    println!("listening on port 9999 for tcp commands");

    for stream in listener.incoming() {
        match stream {
            Ok(stream) => {
                thread::spawn(move || handle_client(&stream));
            }
            Err(e) => {
                eprintln!("err with tcp conn: {}", e);
            }
        }
    }
}

fn handle_client(mut stream: &TcpStream) {
    for line in BufReader::new(stream).lines() {
        let buffer = match line {
            Ok(data) => data,
            Err(_) => return,
        };

        if buffer.is_empty() {
            break;
        }

        let mut parts = buffer.trim().split_whitespace();
        if let Some(command) = parts.next() {
            let arguments: Vec<&str> = parts.collect();
            let response = handle_command(command, arguments);
            stream.write_all(response.as_bytes()).unwrap();
        } else {
            stream.write_all("invalid command".as_bytes()).unwrap();
        }
    }
}

fn handle_command(command: &str, arguments: Vec<&str>) -> String {
    match command {
        "banport" => {
            if let Some(port) = arguments.get(0) {
                let iptables = iptables::new(false).unwrap();
                let _ = iptables.append_unique(
                    "filter",
                    "INPUT",
                    &format!("-p tcp --dport {} -j DROP", port),
                );

                format!("banned port {} for all ips", port)
            } else {
                "missing args for banport: port".to_string()
            }
        }
        "unbanport" => {
            if let Some(port) = arguments.get(0) {
                let iptables = iptables::new(false).unwrap();
                let _ = iptables.delete(
                    "filter",
                    "INPUT",
                    &format!("-p tcp --dport {} -j DROP", port),
                );

                format!("unbanned port {}", port)
            } else {
                "missing args for unbanport: port".to_string()
            }
        }
        "allowipport" => {
            if let (Some(ip), Some(port)) = (arguments.get(0), arguments.get(1)) {
                let iptables = iptables::new(false).unwrap();
                let _ = iptables.append_unique(
                    "filter",
                    "INPUT",
                    &format!("-s {} -p tcp --dport {} -j ACCEPT", ip, port),
                );
                format!("allowed {} to access {}", ip, port)
            } else {
                "missing args for allowipport: ip and port".to_string()
            }
        }
        "removeipport" => {
            if let (Some(ip), Some(port)) = (arguments.get(0), arguments.get(1)) {
                let iptables = iptables::new(false).unwrap();
                let _ = iptables.delete(
                    "filter",
                    "INPUT",
                    &format!("-s {} -p tcp --dport {} -j ACCEPT", ip, port),
                );
                format!("rm {} access to {}", ip, port)
            } else {
                "missing args for rmipport: ip and port".to_string()
            }
        }
        _ => {
            format!("unknown command: {}", command)
        }
    }
}
